package tangram

import tangram.http.HTTPSClient
import tangram.http.TangramHttpRequest

def worker = new CliProductsWorker(request, sql, json)
worker.run()

/**
 * PPG ClientLink interface - Product send (skript)
 *   COUNTRY=CZ alebo SK
 *   TEST=TRUE alebo FALSE (default)
 *   MAX=nnn (max.pocet exportovanych zaznamov, default=1000)
 * Export dat z tabulky TOVARY, pridane stlpce X_CRM, X_CRMID, X_CRMSENT, X_CRMRESULT
 * Pre kazdy tovar sa vytvori samostatna SOAP-sprava a odosle sa na server
 */
class CliProductsWorker
{
    // Premenne naplnene constructorom
    TangramHttpRequest httpRequest
    SQLconn sql
    TangramJsonWriter json
    // Parametre requestu
    String country
    boolean test
    int max = 0
    // Pripojenie do ClientLink cez SOAP
    String url
    String user
    String password
    String template

    // Constructor
    CliProductsWorker(TangramHttpRequest httpRequest, SQLconn sql, TangramJsonWriter json)
    {
        this.httpRequest = httpRequest
        this.sql = sql
        this.json = json

        Log.info("Client Link - Products export")
        def params = this.httpRequest.queryMap
        country = params.getString('country')
        test = params.getBoolean('test', true)
        max = params.getInt('max', 1000)
        if (country!='SK' && country!='CZ' && country!='XX')
            throw new TangramException('Bad request, required parameter COUNTRY=CZ or COUNTRY=SK')

        // Mapovanie SK->PPGSVK, CZ->TRIGA, budeme potrebovat nastavit rozne hodnoty pre SK/CZ a pre TEST/PROD
        String schema = ''
        if (country=='CZ') schema = test ? 'TESTCZ' : 'TRIGA'
        if (country=='SK') schema = test ? 'TESTSK' : 'PPGSVK'
        if (country=='XX') schema = 'EKO'
        def info = "Setting schema to $schema"
        Log.info(info)
        try { sql.conn.setSchema (schema) }
        catch (Exception e) { throw new TangramException(info, e) }

        // nastavenie pripojenia do CLI
        // def url = "https://e100211-iflmap.hcisbt.eu1.hana.ondemand.com:443/cxf/Tangram/product/dev"
        url = test ?
                'https://e100211-iflmap.hcisbt.eu1.hana.ondemand.com:443/cxf/Tangram/product/qa' :
                'https://l100431-iflmap.hcisbp.eu1.hana.ondemand.com:443/cxf/Tangram/product/prd'
        user     = test ? 'S0021493398' : 'S0021493400'
        password = test ? 'Um%`.2Jr' : '!z96f$CJ'
        template = (country=='CZ') ? 'PPG/CliProductCZ.ftlx' : 'PPG/CliProductSK.ftlx'
    }

    // Odoslanie vsetkych produktov s novsim timestampom
    void run ()
    {
        // Vyhladanie tovarov: priznak X_crm musi byt zapnuty
        def select = """
        SELECT T.Xx, T.X_crmid, T.Druh, T.Nazov, T.Pozn, T.Mj, nvl(T.Jmj,T.Mj) as Jmj, nvl(T.Jkoef,1) as Jkoef,
            count(*) over() as Pocet,
            (select max(Barcode) from tovbarcode where Kod=T.Xx and Hlavny is not null) as Barcode
        FROM TOVARY T
        WHERE (T.X_crm is not null) and (T.X_crmsent is null or nvl(T.Xud,T.Xid) > T.X_crmsent)
        ORDER BY T.Xx
        """.stripIndent()

        def rs = sql.queryRows(select)

        // Generovanie odpovede pre volajuceho, po kazdom zazname dopiseme a flushneme aby nedoslo k timeoutu
        httpRequest.sendHeaderJSON()
        json.begin()

        // Teraz otvorit response header a po kazdom sendProductMsg zapisat do response body, aby nedoslo k timeoutu pri volani
        def counter = 1
        while (rs.next())
        {
            def xx = rs.getString("xx")
            def pocet = rs.getInt("pocet")
            rs.data.xmj = convertUnit(rs.getString("mj"))
            rs.data.xjmj = convertUnit(rs.getString("jmj"))
            rs.data.xcategory = rs.getString("druh")
            if (rs.getString('barcode').length() != 13) rs.data.barcode=''  // posielame len 13-miestne kody
            Log.newline()
            Log.info("Sending PRODUCT message ($counter/$pocet), productID=$xx")
            Log.debug("${rs.data}")

            // Odoslanie jedneho produktu
            sendProductMsg(rs.data)

            // po kazdom tovare pridame riadok do vystupu, aby klient nevyhodil timeout
            json.pair(xx, rs.getString('nazov'))
            json.flush()

            // kontrola maximalneho poctu zaznamov
            counter++
            if (counter > max) break
        }
        rs.close()

        json.end()

        // Ak nebola ziadna firma odoslana, tak zapiseme hlasku do logu
        if (counter==1)
            Log.info("No new or modified PRODUCT found")
    }

    // Odoslanie jedneho produktu, formatovanie spravy cez FreeMarker (Object HTML)
    private sendProductMsg(DataBuffer data)
    {
        def timeStart = System.currentTimeMillis()
        def body = HTML.process(data, template)
        def timeTemplate = System.currentTimeMillis()

        // Odoslanie zaznamu do ClientLink
        String response = sendSOAP(body)
        def timeSending = System.currentTimeMillis()
        def xx = data.getString('xx')
        Log.info("PRODUCT message sent, ID=$xx, formatting=${timeTemplate-timeStart}ms, sending=${timeSending-timeTemplate}ms")

        // Zapis odpovede do nasej databazy
        saveResponse(response, xx)
    }

    // Odoslanie spravy na HTTP server (body obsahuje naformatovanu SOAP spravu)
    // Vracia Response-body, aby sme mohli analyzovat vysledok
    private String sendSOAP (String body)
    {
        Log.debug("HTTP REQUEST:$url\n$body")
        // PPG CRM connect
        // TODO >> mozne zrychlenie, ak HTTPS klienta inicializujeme len raz na zaciatku ?
        def client = new HTTPSClient(url)
        client.setHeader("Accept", "text/xml")
        client.setHeader("Content-Type", "text/xml;charset=utf-8")
        client.setHeader("User-Agent", "Tangram")
        client.setBasicAuth(user, password)
        // Odoslanie
        client.post(body)
        //
        Log.debug("HTTP RESPONSE:\n${client.responseText}")
        Log.debug("RESPONSE HEADERS:\n${client.connection.headerFields}")
        return client.responseText
    }

    // Konverzia mernych jednotiek na zvlastny PPG ciselnik, default je ZPL (piece/kus)
    private static String convertUnit(String mj)
    {
        switch (mj.toUpperCase())
        {
            case 'KS': return 'Z65'
            case 'KG': return 'KGM'
            case 'L':  return 'LTR'
            case 'BAL':return 'XPK'
            default: return 'Z65'
        }
    }

    // Zapis vysledku do databazy
    private saveResponse (String response, String tovarXX)
    {
        def xml = new TangramXmlReader()
        def idTangram = xml.getTag(response, "ReferenceObjectNodeSenderTechnicalID")
        def idC4C = xml.getTag(response, "C4CInternalID")
        def errors = xml.getTags(response, "ErrorNote")
        Log.info("SAVE RESPONSE TO DATABASE TangramId=$idTangram, C4CId=$idC4C, ERRORS=$errors")
        // OK alebo ERROR ?
        def prm = new DataBuffer([xx: tovarXX, crmid: idC4C, errors: errors.join(';').take(250)])
        if (idC4C)
            sql.execute ("UPDATE TOVARY SET X_crmid=:crmid, X_crmsent=sysdate, X_crmresult=:errors WHERE Xx=:xx", prm)
        else
            sql.execute ("UPDATE TOVARY SET X_crmsent=sysdate, X_crmresult=:errors WHERE Xx=:xx", prm)
    }

}
